           (*-------------------------------------------*
            |  The Dining Mathematicians in Csp-Prover  |
            |                June 2004                  |
            |             December 2004 (modified)      |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory DM14_Spc_def = DM13_Seq_Sys:

(*****************************************************************

         1. defines Spc 
         2.
         3. 
         4. 

 *****************************************************************)

(*********************************************************
                     specification 
 *********************************************************)

datatype SpcName = TH0_TH1 | EAT0_TH1 | TH0_EAT1

consts
  SpcDef :: "(SpcName, Event) procDF"

primrec
   "SpcDef
      TH0_TH1  = ! x:OBS -> IF (x = Eat0) THEN <EAT0_TH1>
                                          ELSE IF (x = Eat1) 
                                               THEN <TH0_EAT1>
                                               ELSE <TH0_TH1>"

   "SpcDef
      EAT0_TH1 = ! x:(OBS - {Eat1}) -> IF (x = End0) 
                                       THEN <TH0_TH1>
                                       ELSE <EAT0_TH1>"

   "SpcDef
      TH0_EAT1 = ! x:(OBS - {Eat0}) -> IF (x = End1) 
                                       THEN <TH0_TH1>
                                       ELSE <TH0_EAT1>"

consts
  Spc :: "(SpcName, Event) procRC"

defs
  Spc_def:
   "Spc == LET SpcDef IN <TH0_TH1>"

(*********************************************************
        Lemmas for   ALL n.  verify SPC <=sf SYS n
 *********************************************************)

(*** relation between SPCdf and SEQdf ***)

consts 
  Spc_to_Seq :: "SpcName => (SeqName, Event) proc"

primrec
  "Spc_to_Seq 
       TH0_TH1 = 
         NUM !! n .. <Seq_TH0_TH1 n>
         |~|
         NUM !! n .. <Seq_Back0_TH1 n>
         |~|
         NUM !! n .. <Seq_TH0_Back1 n>
         |~|
         NUM !! n:EVENs .. <Seq_Eat0_TH1 n>
         |~|
         NUM !! n:ODDs  .. <Seq_TH0_Eat1 n>
         |~|
         NUM !! n:EVENs .. <Seq_WR0_TH1 n>
         |~|
         NUM !! n:EVENs .. <Seq_WR0_Back1 n>
         |~|
         NUM !! n:ODDs  .. <Seq_TH0_WR1 n>
         |~|
         NUM !! n:ODDs  .. <Seq_Back0_WR1 n>"

  "Spc_to_Seq 
       EAT0_TH1 = 
         NUM !! n:EVENs .. <Seq_EAT0_TH1 n>
         |~|
         NUM !! n:EVENs .. <Seq_EAT0_Back1 n>"

  "Spc_to_Seq 
       TH0_EAT1 = 
         NUM !! n:ODDs .. <Seq_TH0_EAT1 n>
         |~|
         NUM !! n:ODDs .. <Seq_Back0_EAT1 n>"

(*********************************************************
               nohide and guarded lemmas
 *********************************************************)

lemma guardSpcDef[simp]: "!!C. guard ((SpcDef) C)"
by (induct_tac C, simp_all)

lemma nohideSpcDef[simp]: "!!C. nohide ((SpcDef) C)"
by (induct_tac C, simp_all)

(************************************************************)

(*** TH0_EAT1 ***)

lemma TH0_EAT1[simp]:
  "LET SeqDef IN (Rewrite (SpcDef TH0_EAT1) By Spc_to_Seq) <=F
   LET SeqDef IN Spc_to_Seq TH0_EAT1"
apply (auto)
apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_dist_tac 1 *})
apply (rule)

(* *)
apply (tactic {* csp_step_right_tac 1 *})
apply (auto intro!: csp_decompo_subset)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (rule)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

(* *)

apply (tactic {* csp_step_right_tac 1 *})
apply (auto intro!: csp_decompo_subset)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

(****)

apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_step_right_tac 1 *})
apply (auto intro!: csp_decompo_subset)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)
done

(*** EAT0_TH1 ***)

lemma EAT0_TH1[simp]:
  "LET SeqDef IN (Rewrite (SpcDef EAT0_TH1) By Spc_to_Seq) <=F
   LET SeqDef IN Spc_to_Seq EAT0_TH1"
apply (auto)

apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_dist_tac 1 *})
apply (rule)

(* *)
apply (tactic {* csp_step_right_tac 1 *})
apply (auto intro!: csp_decompo_subset)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (rule)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

(* *)

apply (tactic {* csp_step_right_tac 1 *})
apply (auto intro!: csp_decompo_subset)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

(****)

apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_step_right_tac 1 *})
apply (auto intro!: csp_decompo_subset)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left1)
apply (rule csp_Int_choice_left2)
apply (rule csp_Rep_int_choice_left)
apply (rule_tac x="NUM x" in exI)
apply (simp)
done

end
